# General information:
# 1) We will assume for the sake of simplicity that the data fulfill all requirements 
# for the use of t tests: Observations are randomly and independently drawn from 
# normally distributed populations of equal variance.
# 2) The 95% CIs refer only to the individual mean (arelational CI) and cannot be used
# directly to draw conclusions about differences between means (i.e., do not use the CIs 
# to compare different contrasts with each other).


# Example of contrast analyses for one-way between-subject data
#
# Stefan Wiens & Mats E. Nilsson, 2016-06-23
# Contact: mats.nilsson@psychology.su.se


# Clear everything -------------------------------------------------------------
rm(list = ls()) # clear memory
graphics.off() # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
#-------------------------------------------------------------------------------


# Set working directory (mydir) ------------------------------------------------
mydir <- 'c:/Users/MATNI/Dropbox/Statspaper with Mats/Supplement/R'
setwd(mydir)
# ------------------------------------------------------------------------------


# Load functions ---------------------------------------------------------------
source('functions_contrasts.r')
# ------------------------------------------------------------------------------


# Data from Howell, 2013, Table 12.1 -------------------------------------------------
# Dependent variable (dv): Morphine tolerance in mice (paw-licking latency)
dv_time <-  c( 3,  5,  1,  8,  1,  1,  4,  9,  2, 12, 13,  6, 10,  7, 11, 19, 
              14,  6, 12,  4, 19,  3,  9, 21, 29, 20, 36, 21, 25, 18, 26, 17, 
              24, 26, 40, 32, 20, 33, 27, 30)
# Independent variable (iv): Group assignment
group <-    c( 1,  1,  1,  1,  1,  1,  1,  1,  2,  2,  2,  2,  2,  2,  2,  2, 
               3,  3,  3,  3,  3,  3,  3,  3,  4,  4,  4,  4,  4,  4,  4,  4, 
               5,  5,  5,  5,  5,  5,  5,  5)
group_names <- c('G1', 'G2', 'G3', 'G4', 'G5')
iv_group <- factor(group,labels = group_names)
# ------------------------------------------------------------------------------


# Screening plots --------------------------------------------------------------
# Boxplot
par(fig = c(0, 1, 0, 1))
boxplot(dv_time ~ iv_group, xlab = 'Group', ylab = 'Time')
# ------------------------------------------------------------------------------


# Define contrast-weight matrix-------------------------------------------------
# One contrast per row,  weights for groups from left to right
# If you run "contrast_result" below with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" below with "standardset = FALSE",
# these contrasts will be computed with the given weights.
coef <-  matrix(c(-1,  0,  1,  0,  0,  # Group 3 - Group 1          
                  -1,  1,  0,  0,  0,  # Group 2 - Group 1  
                  -2, -1,  0,  1,  2,  # Linear trend
                  -2, -2, -2,  3,  3,  # Groups 4,5 - Group 1,2,3 
                   0, -1,  0,  0,  1,  # Group 5 - Group 2 
                   0,  1, -1,  0,  0), # Group 2 - Group 3          
                   ncol = 5, byrow = TRUE)

# Check if contrast sum to zero (within 1e-12 margin of error)
if (abs(sum(apply(coef,1,sum))) > 1e-12) stop ('Not all contrasts sum to zero!')

# Add row and column names to the contrast-weight matrix 
cnames <- c('3>1', '2>1','Linear','4:5>1:3', '5>2', '2>3')
rownames(coef) <- cnames
colnames(coef) <- levels(iv_group)
#-------------------------------------------------------------------------------


# Calculate contrast scores with CIs, by using ci_contrast() -----------------------
# Below, the error = 'anova2' will pool the error only from groups that are  
# included in the contrast (i.e., from groups that have nonzero weights).
# The error = 'anova' will pool the error from all groups 
# (even if they have zero weights). This is appropriate if all groups have 
# similar variances.
# See below for an example.
contrast_result <- apply(coef, 1, get_bcontrast, data = dv_time, group = iv_group,
                         error = 'anova2', standardset = TRUE)
# If you run "contrast_result" with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" with "standardset = FALSE",
# these contrasts will be computed with the given weights.

contrast_result <- t(contrast_result)  # Transpose matrix t(), if you prefer
                                       # to have each contrasts in a single row
print(contrast_result)
#-------------------------------------------------------------------------------


# Plot contrast with CI --------------------------------------------------------
par(fig = c(0, 1, 0, 1))
plot_contrast(contrast_result)

#-------------------------------------------------------------------------------
# Example results:
#
# contrast_result <- apply(coef, 1, get_bcontrast, data = dv_time, group = iv_group,
#                          error = 'anova2', standardset = TRUE)
#
# Note that the dfs and MSerror are different for all contrasts. The reason is that 
# with "error=anova2", the error is computed only from the groups that have nonzero weights.
# Note further that the contrast weights are turned into standard sets (standardset = TRUE).
# Thus, the contrast score is exactly in the units of the scale.
#
#           Score     CI_LL     CI_UL      Tobs  df            p       SE  MSerror     G1     G2
# 3>1      7.00000  1.369023 12.630977  2.666235 14 1.843509e-02 2.625425 27.57143 -1.000  0.000
# 2>1      6.00000  1.432201 10.567799  2.817270 14 1.370625e-02 2.129722 18.14286 -1.000  1.000
# Linear  21.33333 17.242620 25.424047 10.682575 28 2.196824e-11 1.997022 28.71429 -0.667 -0.333
# 4:5>1:3 18.16667 14.460214 21.873120  9.950293 35 9.657362e-12 1.825742 32.00000 -0.333 -0.333
# 5>2     19.00000 12.920101 25.079899  6.702570 14 1.006318e-05 2.834734 32.14286  0.000 -1.000
# 2>3     -1.00000 -7.408776  5.408776 -0.334664 14 7.428397e-01 2.988072 35.71429  0.000  1.000
#             G3    G4    G5
# 3>1      1.000 0.000 0.000
# 2>1      0.000 0.000 0.000
# Linear   0.000 0.333 0.667
# 4:5>1:3 -0.333 0.500 0.500
# 5>2      0.000 0.000 1.000
# 2>3     -1.000 0.000 0.000

